"""
 Take Ownership: https://learn.microsoft.com/en-us/rest/api/power-bi/reports/take-over-in-group
"""
import typing as t
from pathlib import Path
from urllib.parse import urljoin

import requests
from azure.identity import (  # ClientSecretCredential,; InteractiveBrowserCredential,
    DefaultAzureCredential,
)

SCOPE = "https://analysis.windows.net/powerbi/api/.default"


class PowerBIClient:
    base_url = "https://api.powerbi.com/v1.0/myorg/ "

    # TODO: if client_id, tenant_id, client_secret provided, use ClientSecretCredential
    def __init__(self):
        # By default: AAD
        self.token = self.get_token()

    # TODO: check "authentification" method in config: oauth, browser, service principal
    @staticmethod
    def get_token():
        # TODO: check if azure-cli is available, if not use InteractiveBrowser
        # azure-cli installed (brew install ...)
        # `az login --allow-no-subscriptions` or see the option with scope in parameters
        credentials = DefaultAzureCredential()
        access_token = credentials.get_token(SCOPE)
        token = access_token.token
        return token

    def dump_token(self, path: t.Union[str, Path] = "."):
        token_path = Path(path) / "token.json"

        token_sub_string = '{{"PBIToken":"{0}"}}'.format(self.token)
        token_str = "accessToken={0};".format(token_sub_string)

        with open(token_path, "w", encoding="utf8") as token_file:
            token_file.write(token_str)

        return token_path

    @property
    def headers(self):
        return {"Authorization": f"Bearer {self.token}"}

    def get(self, endpoint, params: t.Optional[dict] = None):
        url = urljoin(self.base_url, endpoint)
        return requests.get(url, params=params, headers=self.headers)

    def get_workspaces(self):
        # TODO: create dataclass to load json
        endpoint = "groups"
        res = self.get(endpoint)
        return res.json()["value"]

    def get_workspace_id(self, workspace_name: str):
        workspaces = self.get_workspaces()
        try:
            return next(
                workspace["id"]
                for workspace in workspaces
                if workspace["name"] == workspace_name
            )
        except StopIteration:
            return None

    def get_reports(self, workspace_id: str) -> dict:
        # TODO: create dataclass to load json
        endpoint = f"groups/{workspace_id}/reports"
        res = self.get(endpoint)
        return res.json()["value"]

    def get_report(self, workspace_id: str, report_name: str) -> dict:
        reports = self.get_reports(workspace_id)
        return next(report for report in reports if report["name"] == report_name)

    def get_report_id(self, workspace_id: str, report_name: str) -> str:
        report = self.get_report(workspace_id, report_name)
        return report["id"]

    def get_pages(self, workspace_id: str, report_id: str) -> list:
        endpoint = f"groups/{workspace_id}/reports/{report_id}/pages"
        print(endpoint)
        res = self.get(endpoint)
        return res.json()["value"]

    def get_page(self, workspace_id: str, report_id: str, page_page: str) -> dict:
        pages = self.get_pages(workspace_id, report_id)
        return next(page for page in pages if page["displayName"] == page_page)

    # https://learn.microsoft.com/en-us/rest/api/power-bi/reports/get-pages-in-group



from pathlib import Path
from shutil import copy

from selenium import webdriver
from selenium.webdriver.chrome.options import Options

HTML_FILENAME = "LoadTest.html"
HTML_FILEPATH = Path(__file__).parent / "static" / HTML_FILENAME


def copy_html(dst_dir) -> Path:
    dst_path = Path(dst_dir) / HTML_FILENAME
    copy(HTML_FILEPATH, str(dst_path))
    return dst_path


def load_url(fileurl: str):
    chrome_options = Options()
    # chrome_options.add_argument("--incognito")
    chrome_options.add_experimental_option("detach", True)
    driver = webdriver.Chrome(options=chrome_options)

    driver.get(fileurl)

    return driver



"""
Functions to create a JSON file ready to used by HTML
"""

import typing as t

import yaml

DEFAULT_CONFIG_PATH = "./config.yaml"


# TODO: use pydantic to load config and validate schema
def load_config(config_path: t.Optional[str] = None):
    if config_path is None:
        config_path = DEFAULT_CONFIG_PATH

    with open(config_path, "r", encoding="utf8") as config_file:
        return yaml.safe_load(config_file)



import json
import time
from pathlib import Path

from selenium.common.exceptions import JavascriptException

from .config import load_config
from .pbi import PowerBIClient
from .scraping import copy_html, load_url


def run(project_dir=None, clean=True):
    """Run the load test"""

    if not project_dir:
        project_dir = Path().cwd()

    pbi_client = PowerBIClient()
    token_path = pbi_client.dump_token(path=project_dir)

    config = load_config()

    workspace_name = config["workspace"]
    print(workspace_name)
    report_name = config["report"]
    print(report_name)
    page_name = config.get("page", "")
    print(page_name)

    # Get Workspace
    workspace_id = pbi_client.get_workspace_id(workspace_name)

    if not workspace_id:
        raise Exception

    print(f"Workspace ID: {workspace_id}")

    # Get Report
    report = pbi_client.get_report(workspace_id, report_name)
    report_url = report["embedUrl"]
    report_id = report["id"]

    print(f"Report ID: {report_id}")

    # Get Page
    page = pbi_client.get_page(workspace_id, report_id, page_name)
    page_id = page["name"]

    print(f"Page ID: {page_id}")

    # Fitlers
    slicers = config.get("slicers", [])
    filters = config.get("filters", [])

    report_path = project_dir / "parameters.json"

    report_dict = {
        "reportUrl": report_url,
        "reportId": report_id,
        "pageName": page_id,
        "sessionRestart": 100,
        "slicers": slicers,
        "filters": filters,
    }

    report_parameter_str = json.dumps(report_dict)

    report_str = "reportParameters={0};".format(report_parameter_str)

    with open(report_path, "w", encoding="utf8") as report_file:
        report_file.write(report_str)

    project_filepath = copy_html(project_dir).absolute()

    fileurl = f"file:///{project_filepath}"

    driver = load_url(fileurl)

    duration = None

    while not duration:
        try:
            duration = driver.execute_script("return duration")

        except JavascriptException:
            time.sleep(1)

    print(f"Duration: {duration}s")

    input("Press Enter to continue...")

    driver.quit()

    if clean:
        print("Start cleaning...")
        Path(report_path).unlink()
        Path(token_path).unlink()
        Path(project_filepath).unlink()



import click

from . import __version__
from .pbi import PowerBIClient
from .run import run as _run


@click.group(invoke_without_command=True)
@click.option("-v", "--version", required=False, is_flag=True)
def cli(version):
    """Application to simplify Power BI Load Test."""
    ctx = click.get_current_context()
    if ctx.invoked_subcommand is None:
        if version:
            click.echo(__version__)
        else:
            click.echo(ctx.get_help())


@cli.command()
def token():
    pbi_client = PowerBIClient()
    click.echo(pbi_client.token, color=True)


@cli.command()
def run():
    _run()
